package metrics

import (
	"time"

	"github.com/prometheus/client_golang/prometheus"
)

func NewStorageTimer(storage, name, implementation string) StorageTimer {
	return StorageTimer{
		storage:        storage,
		name:           name,
		implementation: implementation,
	}
}

type StorageTimer struct {
	storage        string
	name           string
	implementation string
}

func (t *StorageTimer) Time(function string) func(success bool) {
	start := time.Now()
	return func(success bool) {
		successS := "0"
		if success {
			successS = "1"
		}
		storageSummary.WithLabelValues(
			t.storage, t.name, t.implementation, function, successS,
		).Observe(time.Since(start).Seconds())
	}
}

var (
	storageSummary = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "xbus_storage_calls",
			Help:       "Storage API calls durations",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
		},
		[]string{"storage", "name", "implementation", "function", "success"})
)

func init() {
	prometheus.MustRegister(storageSummary)
}
