// Copyright 2016 Orus.io. All rights reserved.

package sql

import (
	"testing"

	"xbus.io/xbus/storage"
	"xbus.io/xbus/testutils"
	"xbus.io/xbus/testutils/sqldb"
	storagetest "xbus.io/xbus/testutils/storage"
)

func envelopeStorageFactoryMaker(tb testing.TB) (func(tb testing.TB) (storage.EnvelopeStorage, func()), func()) {
	tb.Helper()

	testDB := sqldb.GetTestDB(tb)
	if testDB == nil {
		return nil, nil
	}
	m, db := testDB.GetModelDB(1)

	return func(tb testing.TB) (storage.EnvelopeStorage, func()) {
			tb.Helper()

			if _, err := db.Engine.DB().Exec("DELETE FROM envelope"); err != nil {
				tb.Fatal(err)
			}
			if _, err := db.Engine.DB().Exec("DELETE FROM event"); err != nil {
				tb.Fatal(err)
			}
			if _, err := db.Engine.DB().Exec("DELETE FROM item"); err != nil {
				tb.Fatal(err)
			}

			return NewEnvelopeStorage(m, db, *testutils.GetTestLogger(tb).Zero()), func() {
			}
		}, func() {
			db.Close()
			testDB.Close()
		}
}

func TestEnvelopeStorage(t *testing.T) {
	factory, clean := envelopeStorageFactoryMaker(t)
	defer clean()
	storagetest.EnvelopeStorageTests(t, factory, storagetest.EnvelopeStorageTestsOptions{
		SupportUnordered:     false,
		SupportParallelStore: false,
	})
}

func BenchmarkEnvelopeStorage(b *testing.B) {
	factory, clean := envelopeStorageFactoryMaker(b)
	defer clean()
	storagetest.EnvelopeStorageBench(b, factory)
}
