package withfallback

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"xbus.io/go-xbus/v3/api"

	"xbus.io/xbus/storage"
	"xbus.io/xbus/testutils"
	"xbus.io/xbus/testutils/mocks"
	storagetest "xbus.io/xbus/testutils/storage"
)

func envelopeStorageFactory(tb testing.TB) (storage.EnvelopeStorage, func()) {
	tb.Helper()

	return NewEnvelopeStorage(
		mocks.NewEnvelopeStorage(testutils.GetTestLogger(tb)),
		mocks.NewEnvelopeStorage(testutils.GetTestLogger(tb)),
	), func() {}
}

func TestEnvelopeStorage(t *testing.T) {
	storagetest.EnvelopeStorageTests(t, envelopeStorageFactory, storagetest.EnvelopeStorageTestsOptions{
		SupportUnordered:     true,
		SupportParallelStore: true,
	})

	t.Run("fallback", func(t *testing.T) {
		primary := mocks.NewEnvelopeStorage(testutils.GetTestLogger(t))
		fallback := mocks.NewEnvelopeStorage(testutils.GetTestLogger(t))

		combined := NewEnvelopeStorage(primary, fallback)

		env1 := api.Envelope{
			ID: api.MustUUIDFromString("b74415ee-83c1-11e6-8f04-133b336287df"),
			EventIDs: []api.UUID{
				api.MustUUIDFromString("c2e373fe-83c1-11e6-a098-c3d72ea43882"),
			},
		}

		env2 := api.Envelope{
			ID: env1.ID,
			Events: []api.Event{
				{
					ID:    api.MustUUIDFromString("c2e373fe-83c1-11e6-a098-c3d72ea43882"),
					Index: 1,
					Type:  "xbus.test",
					Items: []api.Item{
						[]byte("Data"),
						[]byte("Data"),
					},
					ItemCount: 2,
					Checksum:  1307176161,
				},
			},
		}

		_, err := fallback.StoreEnvelope(env1)
		assert.NoError(t, err)

		status, err := combined.ReceptionStatus(env1.ID)
		assert.NoError(t, err)
		assert.Equal(t, storage.EnvelopeStorageReceptionReceiving, status)

		_, err = fallback.StoreEnvelope(env2)
		assert.NoError(t, err)

		status, err = combined.ReceptionStatus(env1.ID)
		assert.NoError(t, err)
		assert.Equal(t, storage.EnvelopeStorageReceptionComplete, status)

		_, _, err = primary.ReadEnvelope(env1.ID, storage.EnvelopeStartPos(), 0)
		assert.ErrorIs(t, err, storage.ErrNoSuchID)

		_, _, err = combined.ReadEnvelope(env1.ID, storage.EnvelopeStartPos(), 0)
		assert.NoError(t, err)
	})
}
